# This file is part of PyFd, a Python interface to the Fd library
#
# Copyright (C) 2020-2021  European Gravitational Observatory
#
# Author list: Franco Carbognani: franco.carbognani@ego-gw.it
#                   Rhys Poulton: poulton@ego-gw.it
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.


import pytest
import PyFd as fd
import PyCm as cm
import ctypes, time
import sys
import os

@pytest.fixture(scope="session", autouse=True)
def fdio():
    fd.CfgReachState(fd.CfgServerIdle)

    filedir = os.path.dirname(os.path.abspath(__file__))
    fake_argv = [b'bla', b'%s/fdio_config.cfg' %filedir, b'my_cm_name']
    arr = (ctypes.c_char_p * len(fake_argv))()
    arr[:] = fake_argv

    fdio = fd.FdIONew(len(arr), arr)
    fd.FdIOParseAndIni(fdio)

    fd.CfgReachState(fd.CfgServerConfigured)
    fd.CfgReachState(fd.CfgServerActive)

    return fdio


cm_item_getter = { # map from CmMessageGetItemType to appropriate getter
    cm.CmMessageItemTail: None,
    cm.CmMessageItemChar: cm.CmMessageGetChar,
    cm.CmMessageItemShort: cm.CmMessageGetShort,
    cm.CmMessageItemInt: cm.CmMessageGetInt,
    cm.CmMessageItemLong: cm.CmMessageGetLong,
    cm.CmMessageItemFloat: cm.CmMessageGetFloat,
    cm.CmMessageItemDouble: cm.CmMessageGetDouble,
    cm.CmMessageItemText: cm.CmMessageGetText,
    cm.CmMessageItemBytes: cm.CmMessageGetBytes,
    cm.CmMessageItemArray: cm.CmMessageGetArray,
}    

class test(object):
    __test__ = False
    def parse_cm_message(message):
        items = []
        while True:
           getter = cm_item_getter[cm.CmMessageGetItemType(message)]
           if getter is None:
              break
           item = getter(message) # error with "cm send ... -i 1.23"
           items.append(item)
        return items


    @cm.CmMessageHandler
    def my_handler(message, sender, serverName):
        print('Received message with type:', cm.CmMessageGetType(message))
        print('sender:', sender)
        print('serverName:', serverName)
        print('payload:', parse_cm_message(message))
        return cm.CmMessageBreak #or cm.CmMessageOk

def test_install_cm_handler(fdio):

    cm.CmMessageInstallHandler(test.my_handler, 'cm_type') 

    frame = fd.FdIOGetFrame(fdio)

    time.sleep(1)
    fd.FrameFree(frame)

def test_write_sms_data(fdio):

    frame = fd.FdIOGetFrame(fdio)
    
    serTime = frame.contents.GTimeS
    fsamp = 1.0 / frame.contents.dt    
    
    prefix = 'test_prefix1'
    data = 'counter 0 pi 3.1415'
    serData = fd.FrSerDataNew(frame, prefix, serTime, data, fsamp)
    try:
        assert serData.contents.data == data
    except AssertionError as e:
        fd.FrameFree(frame)
        raise AssertionError(e)

    
    prefix = 'test_prefix2'
    data = 'counter 1 bla 123.456' 
    serData = fd.FrSerDataNew(frame, prefix, serTime, data, fsamp)
    try:
        assert serData.contents.data == data
    except AssertionError as e:
        fd.FrameFree(frame)
        raise AssertionError(e)

    time.sleep(1)
    fd.FdIOPutFrame(fdio, frame) # no need for FrameFree?
    
    fd.CfgMsgAddUserInfo("gps of current frame %d s", serTime)
    fd.CfgMsgSendWithTimeout(.001)



